% (c) Mario Hlawitschka 2010
% script to calculate the FNM for a given input field stored in
% fig3_pressure


%crop=true
crop=false

%mode = 'cuda'
%mode = 'Hu'
mode = 'Hu2'
%mode = 'hu'
%mode = 'huC++'
%mode = 'HuC++'
%restriction = 'n'
restriction = 'a'   % somehow, the filter is too much a low-pass, has to be checked!

%D = 0.09; %0.1531; % 15cm so we have 512^2
%D=0.27
D=0.09

f0=1e6;

define_media;

a = size(fig3_pressure,1);
width = 0.09; % 9cm
halfwidth = width/2;
delta = width/a;

Nn=uint32(floor(D/delta));
N = double(Nn)


lambda = lossless.soundspeed/f0;
k=2*pi/lambda;



figure(1)
x=-halfwidth:delta:halfwidth-delta;
y=-halfwidth:delta:halfwidth-delta;
x=x*100;
y=y*100;
%y=0:.1:30-.1;
mesh(y,x,(abs(fig3_pressure)));
view(-68,62);
axis tight;
ylabel('x(cm)');
xlabel('y(cm)');
zlabel('Normalized pressure');
title('original input (FNM)')




plane = zeros(N,N);

%plane(D/2-a+1:D/2+a,D/2-a+1:D/2+a) = fig3_pressure(:,:);
plane(floor(N/2+1-a/2):floor(N/2+a/2),floor(N/2+1-a/2):floor(N/2+a/2)) = fig3_pressure(:,:);


figure(2)
x=-halfwidth*N/a:delta:halfwidth*N/a-delta;
y=-halfwidth*N/a:delta:halfwidth*N/a-delta;
%y=0:.1:30-.1;
x=x*100;
y=y*100;
mesh(y,x,(abs(plane)));
view(-68,62);
axis tight;
title( 'padded input' );
ylabel('x(cm)');
xlabel('y(cm)');
zlabel('Normalized pressure');


uFFT = fft2(plane);

figure(3)
dk = 1/k; % is this the way to set dk?
x=-N/2*dk:dk:(N/2-1)*dk;
y=-N/2*dk:dk:(N/2-1)*dk;
%y=0:.1:30-.1;
mesh(y,x,(abs(uFFT)));
view(-68,62);
axis tight;
title( 'padded input (fourier space)' );
ylabel('kx(?cm)');
xlabel('ky(?cm)');
zlabel('Normalized pressure');

dz = 1e-3; %lambda/5; %0.015; %0.099; %lambda/5; %0.099; %lambda/5; % 3mm in z direction

tic;
step = 1;
layers = 150;
if(crop)
    field = zeros(a,a,layers/step);
else
    field = zeros(N,N,layers/step);
end
dk = 2*pi/(N*delta);

Hu = zeros(N,N);
hu = zeros(N,N);

%kx_a = -pi/delta:dk:pi/delta-dk;
ky_a = -pi/delta:dk:pi/delta-dk;

if(~strcmp(mode, 'cuda'))
    
for layer=1:step:layers+1

    layer
    z=layer*dz;

	% window boundary (squared)
    kc2 = k^2*((D^2)/2/((D^2)/2+z^2));

    switch( mode )
        case 'Hu' % spectral approach

				for m=1:N

					kx = -pi/delta + dk*m - dk;
					for n=1:N
						ky = -pi/delta + dk*n;

                        kxy = kx^2 + ky^2;
						
                        if(abs(kx^2 + ky^2 - k^2 ) < 1e-3)
                            Hu(m,n)=complex(0,0);
                        else
                            if(kxy <= k^2)
                                t1 = sqrt(k^2 - kxy);
                                Hu(m,n) = exp(complex(0.,-z*t1))/complex(0,t1);
                            else
                                t1 = sqrt(kxy - k^2);
                                Hu(m,n) = complex(exp(-z*t1)/t1,0.0);
                            end
                        end
                        
                        if(restriction == 'a')
                            if(kx^2 + ky^2 > kc2)
                                Hu(m,n) = 0;
                            end
                        end
						%Hu(m,n) = exp(-z*sqrt(kx^2 + ky^2 - k^2))/sqrt(kx^2 + ky^2 - k^2);
					end

					%Hu(m,:) = exp(-z*sqrt(ky_a.^2 + (kx^2 - k2)))./sqrt(ky_a.^2 + (kx^2 - k2));
                end
			Hu = fftshift(fftshift(Hu,1),2);

        case 'hu' % spatial approach
            for i=1:N
                % disp(i)
                x = (-N/2 + i)*delta;
                for j=1:N
                    y = (-N/2 + j)*delta;
                    r = sqrt(x^2 + y^2 + z^2);
                    hu(i,j) = exp(-1j*k*r)/(2*pi*r);
                end
            end
            Hu = fft2(fftshift(fftshift(hu,1),2));

        case 'huC++'
                for m=1:N
                    x = (-N/2 + m)*delta;
                    for n=1:N
                        y = (-N/2 + n)*delta;
                        hu(m,n) = MLAngularSpectrumHuSpatial( [x,y,z], k );
                    end
                end
                Hu = fft2(fftshift(fftshift(hu,1),2));

        case 'Hu2'

                for m=1:N

					kx = (-pi/delta + dk*m - dk)/k;
					for n=1:N
						ky = (-pi/delta + dk*n)/k;

                        kxy = kx^2 + ky^2;
                        if(abs(kxy - 1 ) < 1e-15)
                            Hu(m,n)=complex(0,0);
                        else
                            if(kxy <= 1)
                                t1 = k*sqrt(1 - kxy);
                                Hu(m,n) = exp(complex(0.,-z*t1))/complex(0,t1);
                            else
                                t1 = k*sqrt(kxy - 1);
                                Hu(m,n) = complex(exp(-z*t1)/t1,0.0);
                            end
                        end
						%Hu(m,n) = exp(-z*sqrt(kx^2 + ky^2 - k^2))/sqrt(kx^2 + ky^2 - k^2);
                        
                        if(restriction == 'a')
                            if(kx^2 + ky^2 > kc2/k^2)
                                Hu(m,n) = 0;
                            end
                        end
					end
                end
                Hu = fftshift(fftshift(Hu,1),2);


        case 'HuC++'
            for m=1:N
                kx = -pi/delta + dk*m - dk;
                for n=1:N
                    ky = -pi/delta + dk*n;

                    Hu(m,n) = MLAngularSpectrumHu([kx, ky], z, k);
                    
                    if(restriction == 'a')
                        if(kx^2 + ky^2 > kc2/k^2)
                            Hu(m,n) = 0;
                        end
                    end
                end
                Hu = fftshift(fftshift(Hu,1),2);
            end
        otherwise
                error('oops');
    end


    
    %toc
    %figure(6)
    %hu = ifft2(Hu);
    %surf(abs(Hu))
    outplane = ifft2( (1j*k*lossless.soundspeed) * uFFT .* Hu ); %fftshift(fftshift(Hu,1),2));
%toc
    if(crop)
        field(:,:,(layer+step-1)/step) = outplane(floor((N-a)/2)+1:floor((N+a)/2),floor((N-a)/2)+1:floor((N+a)/2));
    else
        field(:,:,(layer+step-1)/step) = outplane(:,:);
    end
    
    %toc
end

else
    %mode == cuda
    tic;
    field = MLAngularSpectrumApproach( uFFT, [N N (layers)/step+1 ], [delta delta dz*step], k);
    toc
    if(crop)
        field = field(floor((N-a)/2)+1:floor((N+a)/2),floor((N-a)/2)+1:floor((N+a)/2),:);
    end
end

figure(4)
x=-N/2*dk:dk:(N/2-1)*dk;
y=-N/2*dk:dk:(N/2-1)*dk;
%y=0:.1:30-.1;
mesh(y,x,(abs(Hu)));
view(-68,62);
axis tight;
title( 'spectral porpagator' );
ylabel('kx');
xlabel('ky');
zlabel('spectral propagator');


out = squeeze(field(:,floor(a/2),:));

figure(5)
if(crop)
    x=-halfwidth:delta:halfwidth-delta;
else
    x=-halfwidth*N/a:delta:halfwidth*N/a-delta;
end
x=x.*100;
%y=-halfwidth*D/a:delta:halfwidth*D/a-delta;
y=0:dz*step:layers*dz;
y=y.*100;
mesh(y,x,(abs(out)));
view(-68,62);
axis tight;
ylabel('x(cm)');
xlabel('y(cm)');
zlabel('output pressure');
title('asa output')

