% (c) Mario Hlawitschka 2010
% script to generate the image from figure 3 from Zeng's ASA paper
% known conditions:
% Transducer: rectangle, 3cmx3cm at origin
% % Pressure Grid is x-z plane, x from -4.5 to 4.5cm, z from 0 to 30cm
% Pressure Grid in s-y plane, x from -4.5 to 4.5cm, y from -4.5 to 4.5 cm
% z = 0.0015m (= 0.15cm)
% f0=1e6, losses, nonattenuating medium, 1500m/s
% piston size in wavelength 20lambda
% Result is normalized, 
f0=1e6;

define_media;
Transducer=get_rect(.015,.015,[0 0 0],[0 0 0]);
zc = 0.0015 % 0.15mm
delta = 0.0006; %0.0003 % 0.3mm
ps=set_coordinate_grid(delta,-.045,.045,-.045,.045,zc,zc);
fig3_pressure=squeeze(gpu_fnm_call(Transducer,ps,lossless,80,f0,0));
fig3_pressure=fig3_pressure./max(max(fig3_pressure));

%trimming pressure field to make it look better
%fig3_pressure=fig3_pressure(1:90,1:90);
x=-4.5:delta*100:4.5;
y=-4.5:delta*100:4.5;
%y=0:.1:30-.1;
mesh(y,x,(abs(fig3_pressure)));
view(-68,62);
axis tight;
ylabel('x(cm)');
xlabel('y(cm)');
zlabel('Normalized pressure');