function Pressure=fnm_call_gpu(xdc,cg,medium,ndiv,f0,dflag)
% This is a wrapper function to call fnm_cw with error checking.
% do NOT call fnm_cw directly or risk segfaults
% this funtion is intended for advanced programming. It is reommended
% that users avoid this function unless they need to run scripts. 
% documentation is provided in the readme files.
% if the call fails, pressure is set to an empty array
if nargin()~=6 && nargin() ~=7
    Pressure=[];
	error('fnm_call requires EXACTLY 6 arguments')
end

if isstruct(xdc)==0
    Pressure=[];
	error('xdc needs to be a struct, please create it')
end
if isstruct(cg)==0
    Pressure=[];
	error('cg needs to be a struct, please use set_coordinate_grid to create it')
end
if isstruct(medium)==0
    Pressure=[];
	error('medium needs to be a struct, please use define_media to create several commonly used medium')
end

if isnumeric(ndiv)==0 || ndiv<1
    Pressure=[];
	error('ndiv must be greater then or equal to 1')
end
if isnumeric(f0)==0 || f0<0
    Pressure=[];
	error('f0 must be greater then or equal to 0')
end
if dflag >=1 
    dflag=1;
else
    dflag=0;
end
if cg.vector_flag==-1
    nx=(cg.xmax-cg.xmin)/cg.delta(1)+1;
    ny=(cg.ymax-cg.ymin)/cg.delta(2)+1;
    nz=(cg.zmax-cg.zmin)/cg.delta(3)+1;
    nobs=nx*ny*nz;
    if nobs>20796875
		sprintf('WARNING: nx= %i, my=%i, nz=%i, total elements=%i',nx,ny,nz,nobs)
        warning('Final answer may exceed the capabilities of a 32-bit system')

        %cont=input('Continue? y/n');
        %if cont=='n' || cont=='N'||cont='no' ||cont==[]
       %     Pressure=[];
       %     error('Aborted by user')
       % end
    end
end
Pressure=fnm_cw_gpu(xdc,cg,medium,ndiv,f0,dflag);

