function Pressure=fnm_cw_cpu(xdc,cg,medium,ndiv,f0,dflag)
% This is a wrapper function to the cuda version of fnm_cw with error checking

if nargin()~=6 % && nargin() ~=7
	Pressure=[];
	error('gpu_fnm_call requires EXACTLY 6 arguments')
end

if isstruct(xdc)==0
	Pressure=[];
	error('xdc needs to be a struct, please create it')
end
if isstruct(cg)==0
	Pressure=[];
	error('cg needs to be a struct, please use set_coordinate_grid to create it')
end
if isstruct(medium)==0
	Pressure=[];
	error('medium needs to be a struct, please use define_media to create several commonly used media')
end

if isnumeric(ndiv)==0 || ndiv<1
	Pressure=[];
	error('ndiv must be greater than or equal to 1')
end
if isnumeric(f0)==0 || f0<0
	Pressure=[];
	error('f0 must be greater than or equal to 0')
end
if dflag >=1
	dflag=1;
else
	dflag=0;
end

% nx = 1
% ny = 1
% nz = 1
% cg.delta(1)
% cg.delta(2)
% cg.delta(3)

if not(cg.delta(1)==0)
    nx = (cg.xmax-cg.xmin)/cg.delta(1)+1;
    dx = cg.delta(1);
else
    nx = 1;
    dx = 1;
end
if not(cg.delta(2)==0)
    ny = (cg.ymax-cg.ymin)/cg.delta(2)+1;
    dy = cg.delta(2);
else
    ny = 1;
    dy = 1;
end
if not(cg.delta(3)==0)
    nz = (cg.zmax-cg.zmin)/cg.delta(3)+1;
    dz = cg.delta(3);
else
    nz = 1;
    dz = 1;
end
nobs=nx*ny*nz;
if nobs>20796875
	sprintf('WARNING: nx=%i, ny=%i, nz=%i, total eleents=%i', nx, ny, nz, nobs)
	warning('Final answer may exceed the capabilities of a 32-bit system')
end

% in case of error, set pressure to an empty array
Pressure=[];

% check for apodization_index = 0 (uniform)
%if xdc.piston_apodization_index~=0
%	error('we only handle a piston_apodization_index of 0, i.e., uniform rectangular and circular pistons so far.');
%end


% Check the medium
if isstruct(medium)==0
	error('A medium must be defined. Please set the medium to lossless medium');
end

% medium.cb         %
% medium.wb         %
% medium.rho 		% density
% medium.c_sound	% speed of sound
% medium.b			%
% medium.atten_coeff%
% medium.ct			%
% medium.kappa		%
% medium.beta		%

lambda = medium.soundspeed/f0;
k = 2*pi/lambda;

% m_factor = medium.rho * medium.c_sound * u0

% cg.delta(1)
% cg.delta(2)
% cg.delta(3)
% cg.xmin
% cg.xmax
% cg.ymin
% cg.ymax
% cg.zmin
% cg.zmax

% translation and scaling of grid in world space
grid_trans = [dx   0             0             0; ...
		 0             dy   0             0; ...
		 0             0             dz   0; ...
		 cg.xmin       cg.ymin       cg.zmin       1];

% translation based on piston position
piston_trans = [1 0 0 0; ...
				0 1 0 0; ...
				0 0 1 0; ...
				-xdc.center(1) -xdc.center(2) -xdc.center(3) 1];

% rotation around piston center (for rotated pistons)
theta = xdc.euler(1);
psi   = xdc.euler(2);
phi   = xdc.euler(3);
costheta = cos(theta);
sintheta = sin(theta);
cospsi = cos(psi);
sinpsi = sin(psi);
cosphi = cos(phi);
sinphi = sin(phi);

A = zeros(4);
A(1,1)= cosphi*costheta-sinphi*sinpsi*sintheta;
A(2,1)= sinphi*costheta+cosphi*sinpsi*sintheta;
A(3,1)= -cospsi*sintheta;
A(1,2)= -sinphi*cospsi;
A(2,2)= cosphi*cospsi;
A(3,2)= sinpsi;
A(1,3)= cosphi*sintheta+sinphi*sinpsi*costheta;
A(2,3)= sinphi*sintheta-cosphi*sinpsi*costheta;
A(3,3)= cospsi*costheta;
A(4,4)= 1;
A;

% transform into the piston's coordinate system
trans = A * piston_trans * grid_trans;


if xdc.shape=='rect'
     Pressure= ...
		MLRectangularPistonGrid(trans, [nx ny nz], 2*xdc.half_height, 2*xdc.half_width, k, 'cpu', ndiv );
else
	if xdc.shape=='circ'
		Pressure= ...
			MLCircularPistonGrid(trans, [nx ny nz], xdc.radius, k, 'cpu', ndiv);
	else
		sprintf('ERROR: piston shape %s cannot be computed using the gpu version, yet.', xdc.shape);
		Pressure=[];
		error('Aborting because of unknown piston type.' )
	end
end

% weight the piston result by the given complex value
Pressure = Pressure;% FIXME!!! * rect_struct.complex_weight;

if xdc.time_delay~=0
	warning('Time delay is not 0 but we do not handle that yet. Set xdc.complex_weight accordingly.');
end
