% reproduces the figure of McGough's paper on the Fast Nearfield Method for Rectangular Pistons
% using four different versions of the code and computes the absolute error between them
%

f = 1540000;
c = 1500;
k = 2*pi*f/c;
lambda = c/f;

s = 5*lambda;
l = 7.5*lambda;

siz=[93 1 93]; %[500 1 200];
%siz = [93*2 1 93*2];
dist=[s/2/siz(1)/2*3    1   s*s/4/lambda/siz(3)];
transform = [ dist(1) 0 0 0; 0 dist(2) 0 0; 0 0 dist(3) 0; 0 0 0 1];
transform = transpose(transform)

out1 = MLRectangularPiston(dist, siz, l, s, k);
out2 = MLRectangularPistonGrid(transform, siz, l, s, k, 'cuda');
out3 = MLRectangularPistonGrid(transform, siz, l, s, k, 'cpu');
out4 = MLRectangularPistonGridCPU(transform, siz, l, s, k);

norm1 = abs(reshape(out1, [(siz(1)) (siz(3))]));
norm2 = abs(reshape(out2, [(siz(1)) (siz(3))]));
norm3 = abs(reshape(out3, [(siz(1)) (siz(3))]));
norm4 = abs(reshape(out4, [(siz(1)) (siz(3))]));


figure(1);
subplot(3,2,1); surf(norm1); title('1) MLRectangularPiston (cuda)');
subplot(3,2,2); surf(norm2); title('2) MLRectangularPistonGrid+cuda');
subplot(3,2,3); surf(norm3); title('3) MLRectangularPistonGrid+cpu');
subplot(3,2,4); surf(norm4); title('4) MLRectangularPistonGridCPU');

subplot(3,2,5); surf(norm1-norm3); title('error 1)-3)');
subplot(3,2,6); surf(norm2-norm3); title('error 2)-3)');

saveas(figure(1), '/tmp/FNM_RectangularPiston_comparison.pdf', 'pdf')
