% fnm.m

% McGough's FNM method
% Dustin E. Kruse
% written 5/1/2008
% updated 8/26/2009
% changed interface 7/26/2010 (mario)
% changed 8/23/2010 (dustin)
function Pressure=DustinRectangularPistonGrid2( transform, siz, l, s, k, nabscissas )

u0=1; % surface velocity
c=1;
rho=1;

[cpx,cpy,cpz]=meshgrid( ...
    dot(transform(:,1),[0 0 0 1]):dot(transform(:,1),[1,0,0,0]):dot(transform(:,1),[siz(1)-1,0,0,1]), ...
    dot(transform(:,2),[0 0 0 1]):dot(transform(:,2),[0,1,0,0]):dot(transform(:,2),[0,siz(2)-1,0,1]), ...
    dot(transform(:,3),[0 0 0 1]):dot(transform(:,3),[0,0,1,0]):dot(transform(:,3),[0,0,siz(3)-1,1]));

cp=[cpx(:),cpy(:),cpz(:)].';

p=complex(zeros(size(cp,1),1));

% find Gauss abscissas and weights
[xx,ww]=gaussquad(nabscissas,-1,1);
% [xx,ww]=grule(nabscissas,3);
xx=xx(:).';
ww=ww(:).';

tmp=complex(0,1)*rho*c*u0/2/pi;

% size(cp,2)

% fnm method for rectangular elements
tic;
for i=1:size(cp,2),

    % disp(num2str(i))
    
    x=cp(1,i);
    y=cp(2,i);
    z=cp(3,i);

    a=s/2; %elem_width_azi/2;
    b=l/2; %elem_width_ele/2;

    s1=a-x;
    l1=b-y;
    s2=a+x;
    l2=b+y;

    
    bb=l2;
    aa=-l1;
    
    if y==0,
        aa=0;
        xx2=(bb-aa)/2*xx+(aa+bb)/2;
        ww2=(bb-aa)/2*ww;
        xsq=xx2.^2;
        term1=2*s1.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s1.^2))-exp(-complex(0,1)*k*z))./(xsq+s1.^2));
    else
        xx2=(bb-aa)/2*xx+(aa+bb)/2;
        ww2=(bb-aa)/2*ww;
        xsq=xx2.^2;
        term1=s1.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s1.^2))-exp(-complex(0,1)*k*z))./(xsq+s1.^2));
    end
    
    %plot(xx2,real(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s1.^2))-exp(-complex(0,1)*k*z))./(xsq+s1.^2)));hold on
    %plot(xx2,imag(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s1.^2))-exp(-complex(0,1)*k*z))./(xsq+s1.^2)),'g');hold off
    %title('1');
    %pause;
    
    bb=s2;
    aa=-s1;
    xx2=(bb-aa)/2*xx+(aa+bb)/2;
    ww2=(bb-aa)/2*ww;
    xsq=xx2.^2;
    term2=l1.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l1.^2))-exp(-complex(0,1)*k*z))./(xsq+l1.^2));
    
    %plot(xx2,real(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l1.^2))-exp(-complex(0,1)*k*z))./(xsq+l1.^2)));hold on
    %plot(xx2,imag(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l1.^2))-exp(-complex(0,1)*k*z))./(xsq+l1.^2)),'g');hold off
    %title('2');
    %pause;
    
    bb=l2;
    aa=-l1;
    if y==0,
        aa=0;
        xx2=(bb-aa)/2*xx+(aa+bb)/2;
        ww2=(bb-aa)/2*ww;
        xsq=xx2.^2;
        term3=2*s2.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s2.^2))-exp(-complex(0,1)*k*z))./(xsq+s2.^2));
    else
        xx2=(bb-aa)/2*xx+(aa+bb)/2;
        ww2=(bb-aa)/2*ww;
        xsq=xx2.^2;
        term3=s2.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s2.^2))-exp(-complex(0,1)*k*z))./(xsq+s2.^2));
    end
    
    %plot(xx2,real(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s2.^2))-exp(-complex(0,1)*k*z))./(xsq+s2.^2)));hold on
    %plot(xx2,imag(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+s2.^2))-exp(-complex(0,1)*k*z))./(xsq+s2.^2)),'g');hold off
    %plot(xx2,1./(xsq+s2.^2),'r');hold off
    %title('3');
    %pause;
    
    bb=s2;
    aa=-s1;
    xx2=(bb-aa)/2*xx+(aa+bb)/2;
    ww2=(bb-aa)/2*ww;
    xsq=xx2.^2;
    term4=l2.*sum(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l2.^2))-exp(-complex(0,1)*k*z))./(xsq+l2.^2));
    
    %plot(xx2,real(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l2.^2))-exp(-complex(0,1)*k*z))./(xsq+l2.^2)));hold on
    %plot(xx2,imag(ww2.*(exp(-complex(0,1)*k*sqrt(z^2+xsq+l2.^2))-exp(-complex(0,1)*k*z))./(xsq+l2.^2)),'g');hold off
    %title('4');
    %pause;
    
    p(i)=tmp*(term1+term2+term3+term4);
end
toc;

p=reshape(p,size(squeeze(cpx)));
Pressure = reshape(p, siz(1), siz(2), siz(3));