

clear all;

c=1500; % speed of sound
f0=1.5e6; % frequency
lambda=c/f0;% wavelength
k=2*pi/lambda; % wave number
%u0=1; % surface velocity
%rho=1000; % density

elem_width_azi=3*lambda; %560e-6; % array element width in azimuth
elem_width_ele=3*lambda; %6e-3; % array element width in elevation
kerf_azi=0; %40e-6; % dead space between array elements in azimuth
% kerf_ele=6.0410/1000;
% kerf_ele=1.0791e-3+4.75/1000;
kerf_ele=0.; %(0.75/2+0.75/2+0.71+4)/1000; % dead space between array elements in elevation
N_azi=7; % number of array elements in azimuth
N_ele=7; % number of array elements in elevation; must stay fixed at 2 for now
% N_azi_math=2;
% N_ele_math=8;

% positive elevation tilt angle
theta1=0; %-10.2*pi/180
% theta1=0;
% negative elevation tilt angle
% theta2=10.2*pi/180;
% theta2=0;

% location of focal point
x_focus=0.000;
y_focus=0.000;
z_focus=-elem_width_azi*(N_azi-1);

% overall width of each array in azimuth and elevation
width_azi=N_azi*elem_width_azi+(N_azi-1)*kerf_azi;
width_ele=N_ele*elem_width_ele+(N_ele-1)*kerf_ele;

% pitch of arrays
pitch=0.; %(width_azi/(N_azi-1))/lambda;

% elevation centers of arrays, pre-tilted
ele_center_1=(N_ele-1)*kerf_ele/2+elem_width_ele/2;
% ele_center_2=-(N_ele-1)*kerf_ele/2-elem_width_ele/2;

% location of the center for each array, pre-tilted
x_center_1=0;
y_center_1=0;% ele_center_1;
z_center_1=0;
center_1=[x_center_1 y_center_1 z_center_1];

% x_center_2=0;
% y_center_2=ele_center_2;
% z_center_2=0;
% center_2=[x_center_2 y_center_2 z_center_2];

% location of focus for each array, pre-tilted
%x_focus_1=x_focus;
%y_focus_1=ele_center_1;
% y_focus_1=y_focus;
%z_focus_1=z_focus;
%focus_1=[x_focus_1 y_focus_1 z_focus_1];
focus_1= [0 0 0.018]

% x_focus_2=x_focus;
% y_focus_2=ele_center_2;
% % y_focus_2=y_focus;
% z_focus_2=z_focus;
% focus_2=[x_focus_2 y_focus_2 z_focus_2];


% centers of physical elements
[xc,yc,zc]=meshgrid((-width_azi/2+elem_width_azi/2):(elem_width_azi+kerf_azi):(width_azi/2-elem_width_azi/2),...
    (-width_ele/2+elem_width_ele/2):(elem_width_ele+kerf_ele):(width_ele/2-elem_width_ele/2),0)


% isolate array elements in positive elevation
%ind1=find(yc>0);
ind1=find(yc > -1e+12); % pick all of them

% translation matrix
T1=[[1 0 0 0];[0 1 0 0];[0 0 1 0];[0 (-kerf_ele/2-elem_width_ele) 0 1]];

X=1;
Y=0;
Z=0;
cc=cos(theta1);
s=sin(theta1);
t=1-cos(theta1);

% rotation matrix
R1=[[t*X^2+cc t*X*Y-s*Z t*X*Z+s*Y 0];
[t*X*Y+s*Z t*Y^2+cc t*Y*Z-s*X 0];
[t*X*Z-s*Y t*Y*Z+s*X t*Z^2+cc 0];
[0 0 0 1]];

% translate and tilt physical elements
tmp=[xc(ind1),yc(ind1),zc(ind1),ones(size(ind1))]*T1*R1*inv(T1);
xct(ind1)=tmp(:,1);
yct(ind1)=tmp(:,2);
zct(ind1)=tmp(:,3);

% tilt surface normal
n1=[0 0 1 1]*R1;
focus_1_tilted=[focus_1 1]*T1*R1*inv(T1);
center_1_tilted=[center_1 1]*T1*R1*inv(T1);


r01cf(ind1)=sqrt((focus_1_tilted(1)-xct(ind1)).^2+(focus_1_tilted(2)-yct(ind1)).^2+(focus_1_tilted(3)-zct(ind1)).^2);
zf(ind1)=r01cf(ind1)-sqrt(sum((focus_1_tilted(1:3)-center_1_tilted(1:3)).^2));

zf=zf(ind1);
zf=zf(:).';
phaseterm=exp(complex(0,1)*k*zf);

phaseterm = reshape(phaseterm, N_azi, N_ele);

phaseterm

%phaseterm =ones(7,7); % for debugging: no focus set
save('phaseterm.mat', 'phaseterm')
