%clear all

FOCUS = true

% demo file for the multiple focus array simulation
% constant parameters
f0 = 1.5e6; % excitation frequency,Hz
% atten = 0; % no attenuation
soundspeed = 1500; % m/s
lambda = soundspeed / f0; % wavelength, m

m_k = 2*pi/(lambda);

%define a transducer structure/array
nelex = 16;
neley = 16;
kerf = 200e-6;
width = 3*lambda-kerf; % transducer width, m
height = 3*lambda-kerf; % transducer height, m
xspacing = width + kerf;
yspacing = height + kerf;

if(FOCUS)
    array_struct=create_rect_planar_array(nelex, neley, width/2, height/2, xspacing, yspacing, [0 0 0]);

    % create the data structure that specifies the attenuation value, etc.
    define_media
end

nx = 96; %129;
ny = 96; %129;
nz = 256;

dx=lambda/4;
dy=lambda/4;
dz=lambda/2;

% define the computational grid
xmin = -(nx-1)*dx;
xmax = 0.0;
ymin = -(ny-1)*dy;
ymax = 0.0;
% zmin = (nz-1)*dz;
% zmax = (nz-1)*dz;

% dx = xmax / (nx - 1);
% dz = zmax / (nz - 1);

% ps = set_coordinate_grid([dx dy dz],xmin,xmax,ymin,ymax,zmin,zmax);

xmtx = [0];
ymtx = [0];
zmtx = [xspacing*nelex];

ndiv = 10;
% calculate the phase for the focus
if(FOCUS)
    array_struct_focus = find_single_focus_phase(array_struct,xmtx,ymtx,zmtx,lossless,f0,ndiv);
end

% generate the reference pressure field
ndiv = 10;
dflag = 0;

% my phaseterm calculation
if(true)
    if(false)
        focus = [ xmtx, ymtx, zmtx ];
        k = 2*pi/lambda;
        centerdist = 0.; %norm(focus(1:3));

        E = nelex;
        eldist = [xspacing yspacing];
        phaseterm = zeros(E,E);
        for l=1:E
            for m=1:E
                ex = (m-1-0.5*(E-1)) * eldist(1);
                ey = (l-1-0.5*(E-1)) * eldist(2);
                phaseterm(m,l) = exp( complex(0., 1) * k * (sqrt((ex-focus(1))^2 + (ey-focus(2))^2 + focus(3)^2)-centerdist));
            end
        end 
    else
        for i=1:nelex*neley
            p(i) = array_struct_focus(i).complex_weight;
            phaseterm(i) = exp( j * m_k * (norm(focus) - norm(focus - array_struct_focus(i).center)) );
        end
    end
end

p=reshape(p, nelex, neley);
diff = phaseterm(8,8) - p(8,8);

figure(1);
imagesc(angle(p));
figure(2);
imagesc(angle(phaseterm+diff));

if(false)
if(FOCUS)
    pref=complex(1,1)*ones(nx,ny,nz);
    for i=1:nz,
        disp(num2str(i));
        ps = set_coordinate_grid([dx dy 0],xmin,xmax,ymin,ymax,(i-1)*dz,(i-1)*dz);
        tic
        pref(:,:,i)=fnm_call(array_struct_focus,ps,lossless,ndiv,f0,dflag);
        toc
    end
else
    m_trans = [ dx 0 0 0; 0 dy 0 0; 0 0 dz 0; xmin ymin 0 1];
    m_focus = [xmtx, ymtx, zmtx];
    % m_focus = complex_weights;
    m_dist = [xspacing, yspacing];
    m_l = width;
    m_s = height;
    pref = MLRectilinearArrayGrid( m_trans, [nx*2, ny*2, nz], m_focus,...
                nelex, neley, m_dist(1), m_dist(2),...
                m_l, m_s, m_k );
end


%normf=complex(1,1)*ones(256,256,256);
%load pref;
%normf(1:129,1:129,1:256)=pref;
%normf(130:256,130:256,:)=pref(128:-1:2,128:-1:2,:);
%clear pref;
%normf(1:129,130:256,:)=normf(1:129,128:-1:2,:);
%normf(130:256,1:129,:)=normf(128:-1:2,1:129,:);
%normf=abs(normf);
normf = abs(pref);
%normf=normf./max(max(max(normf)));
%figure;mesh(abs(squeeze(normf(:,:,100))))

%normf=voxels;

figure;
iso =  0.5;

p=patch(isosurface(normf,iso));
p2=patch(isocaps(normf,iso),'FaceColor','interp','EdgeColor','none');
% isonormals(normf,p);
% colormap(jet(256))
% colorbar

set(p,'FaceColor','red','EdgeColor','none');
camlight headlight;
camlight;
lighting gouraud;
box on
axis([1 256 1 256 1 256])
daspect([1 1 0.5])

close hidden all

if false
% Colors for plotting
red=[1.0 0.0 0.0];  
green=[0.5 1.0 0.5];  
cyan=[0.0 1.0 1.0];  
blue=[0.0 0.0 1.0]; 
yellow=[1.0 1.0 0.0];  
magenta=[1.0 0.0 1.0];
orange=[1.0 0.5 0.0];  
brown=[1.0 0.5 0.5]; 
purple=[0.5 0.5 1.0]; 
black=[0.0 0.0 0.0]; 
noclr='none';

voxels=normf.*double(normf>=0.5);

voxid=1;

% Scale factors for scaling each dimension
sf=[1 1 1];

% Offset of crd of center of voxel(1,1,1) from global origin
offset=[0 0 0];  % center of voxel(1,1,1) = global origin

facecolor=[  % color of each domain
red
green
cyan
blue
yellow
]; 

% edgecolor='none';
edgecolor=[0 0 0];  % black

name='Surface';

fighndl=[];

% -------------------------------------------------------------------------
% Create surfaces for each voxel domain
% In this example faces and vertex are overwritten
% Store them in a cell array or structure if it is required to process them

for id=voxid
    id
    
    % Create surface from voxels
    [faces,vertex]=voxel_bnd_faces(voxels,sf,offset,id);
    
    % Plot surface
    [fighndl]=plotsurf(faces,vertex,facecolor(id,:),edgecolor,name,fighndl);  
        
end
end

end