function Pressure=gpu_fnm_call(xdc,cg,medium,ndiv,f0,dflag)
% This is a wrapper function to the cuda version of fnm_cw with error checking

if nargin()~=6 % && nargin() ~=7
	Pressure=[];
	error('gpu_fnm_call requires EXACTLY 6 arguments')
end

if isstruct(xdc)==0
	Pressure=[];
	error('xdc needs to be a struct, please create it')
end
if isstruct(cg)==0
	Pressure=[];
	error('cg needs to be a struct, please use set_coordinate_grid to create it')
end
if isstruct(medium)==0
	Pressure=[];
	error('medium needs to be a struct, please use define_media to create several commonly used media')
end

if isnumeric(ndiv)==0 || ndiv<1
	Pressure=[];
	error('ndiv must be greater than or equal to 1')
end
if isnumeric(f0)==0 || f0<0
	Pressure=[];
	error('f0 must be greater than or equal to 0')
end
if dflag >=1
	dflag=1;
else
	dflag=0;
end

nx = (cg.xmax-cg.xmin)/cg.delta(1)+1;
ny = (cg.ymax-cg.ymin)/cg.delta(2)+1;
nz = (cg.zmax-cg.zmin)/cg.delta(3)+1;
nobs=nx*ny*nz;
if nobs>20796875
	sprintf('WARNING: nx=%i, ny=%i, nz=%i, total eleents=%i', nx, ny, nz, nobs)
	warning('Final answer may exceed the capabilities of a 32-bit system')
end
                                  

% in case of error, set pressure to an empty array
Pressure=[];

Pressure = gpu_fnm_cw(xdc,cg,medium,ndiv,f0,dflag);
